from __future__ import absolute_import, unicode_literals

from django.db.models import F
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404

from feincms_banners.models import Banner


def click(request, code):
    banner = get_object_or_404(Banner.objects.active(), code=code)
    banner.click(request)
    return HttpResponseRedirect(banner.url)


def impression(request, code):
    if (request.is_ajax() and Banner.objects.filter(code=code).update(
            impressions=F('impressions') + 1
            )):
        return HttpResponse('+1')
    return HttpResponse('?')



from __future__ import absolute_import, unicode_literals

from django.db import models
from django.db.models import F
from django.template.loader import render_to_string
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from feincms_banners.models import Banner

from feincms.admin.item_editor import FeinCMSInline


class BannerContentInline(FeinCMSInline):
    raw_id_fields = ('specific',)


class BannerContent(models.Model):
    feincms_item_editor_inline = BannerContentInline

    is_section_aware = True

    specific = models.ForeignKey(
        Banner, verbose_name=_('specific'),
        blank=True, null=True, limit_choices_to={'is_active': True},
        help_text=_(
            'If you leave this empty, a random banner will be selected.'))
    type = models.CharField(
        _('type'), max_length=20, choices=Banner.TYPE_CHOICES)

    class Meta:
        abstract = True
        verbose_name = _('banner')
        verbose_name_plural = _('banners')

    def render(self, **kwargs):
        if self.specific:
            specific_is_active = (
                self.specific.is_active
                and self.specific.active_from <= timezone.now()
                and (
                    not self.specific.active_until
                    or self.specific.active_until >= timezone.now()
                ))

            if specific_is_active:
                banner = self.specific
                type = banner.type
            else:
                return ''
        else:
            try:
                banner = Banner.objects.active().filter(
                    type=self.type,
                ).select_related('mediafile').order_by('?')[0]
                type = self.type
            except IndexError:
                return ''

        Banner.objects.filter(id=banner.id).update(embeds=F('embeds') + 1)

        return render_to_string(
            [
                'content/banner/%s.html' % type,
                'content/banner/default.html',
            ],
            {'content': self, 'banner': banner},
            context_instance=kwargs.get('context'))



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Banner'
        db.create_table('feincms_banners_banner', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('is_active', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('mediafile', self.gf('feincms.module.medialibrary.fields.MediaFileForeignKey')(to=orm['medialibrary.MediaFile'])),
            ('url', self.gf('django.db.models.fields.URLField')(max_length=200)),
            ('type', self.gf('django.db.models.fields.CharField')(max_length=20)),
            ('code', self.gf('django.db.models.fields.CharField')(default='-z0g3hknx89_h83ko8ybt2wb3s_8tdsiy9hdwpw0', unique=True, max_length=40)),
            ('active_from', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('active_until', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('embeds', self.gf('django.db.models.fields.PositiveIntegerField')(default=0)),
            ('impressions', self.gf('django.db.models.fields.PositiveIntegerField')(default=0)),
        ))
        db.send_create_signal('feincms_banners', ['Banner'])

        # Adding model 'Click'
        db.create_table('feincms_banners_click', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('banner', self.gf('django.db.models.fields.related.ForeignKey')(related_name='clicks', to=orm['feincms_banners.Banner'])),
            ('timestamp', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('ip', self.gf('django.db.models.fields.IPAddressField')(max_length=15, null=True, blank=True)),
            ('user_agent', self.gf('django.db.models.fields.TextField')(default='', blank=True)),
            ('referrer', self.gf('django.db.models.fields.TextField')(default='', blank=True)),
        ))
        db.send_create_signal('feincms_banners', ['Click'])


    def backwards(self, orm):
        # Deleting model 'Banner'
        db.delete_table('feincms_banners_banner')

        # Deleting model 'Click'
        db.delete_table('feincms_banners_click')


    models = {
        'feincms_banners.banner': {
            'Meta': {'ordering': "['-active_from']", 'object_name': 'Banner'},
            'active_from': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'active_until': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'code': ('django.db.models.fields.CharField', [], {'default': "'7u186cw9yz-nbyxc56kaua_e5elyurd7642pb14o'", 'unique': 'True', 'max_length': '40'}),
            'embeds': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'impressions': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'mediafile': ('feincms.module.medialibrary.fields.MediaFileForeignKey', [], {'to': "orm['medialibrary.MediaFile']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'})
        },
        'feincms_banners.click': {
            'Meta': {'ordering': "['-timestamp']", 'object_name': 'Click'},
            'banner': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'clicks'", 'to': "orm['feincms_banners.Banner']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ip': ('django.db.models.fields.IPAddressField', [], {'max_length': '15', 'null': 'True', 'blank': 'True'}),
            'referrer': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'timestamp': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'user_agent': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'})
        },
        'medialibrary.category': {
            'Meta': {'ordering': "['parent__title', 'title']", 'object_name': 'Category'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': "orm['medialibrary.Category']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '150'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '200'})
        },
        'medialibrary.mediafile': {
            'Meta': {'ordering': "['-created']", 'object_name': 'MediaFile'},
            'categories': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': "orm['medialibrary.Category']", 'null': 'True', 'blank': 'True'}),
            'copyright': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '255'}),
            'file_size': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '12'})
        }
    }

    complete_apps = ['feincms_banners']


from __future__ import absolute_import, unicode_literals

from django.conf.urls import url, patterns


urlpatterns = patterns(
    'feincms_banners.views',
    url(r'^b/c/(?P<code>[^/]+)/$', 'click', name='banner_click'),
    url(r'^b/i/(?P<code>[^/]+)/$', 'impression', name='banner_impression'),
)



from __future__ import absolute_import, unicode_literals

from random import choice

from django.db import models
from django.db.models import Q
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from feincms.module.medialibrary.fields import MediaFileForeignKey
from feincms.module.medialibrary.models import MediaFile


def generate_key():
    return ''.join([
        choice('abcdefghijklmnopqrstuvwxyz0123456789-_') for i in range(40)])


class BannerManager(models.Manager):
    def active(self):
        return self.filter(
            Q(is_active=True, active_from__lte=timezone.now())
            & (
                Q(active_until__isnull=True)
                | Q(
                    active_until__isnull=False,
                    active_until__gte=timezone.now()
                )
            )
        )


class Banner(models.Model):
    SKYSCRAPER = 'skyscraper'
    LEADERBOARD = 'leaderboard'
    BOX = 'box'

    TYPE_CHOICES = (
        (SKYSCRAPER, _('skyscraper')),
        (LEADERBOARD, _('leaderboard')),
        (BOX, _('box')),
    )

    is_active = models.BooleanField(_('is active'), default=True)
    name = models.CharField(
        _('name'), max_length=100, help_text=_(
            'Only for internal use, will not be shown on the website.'))
    mediafile = MediaFileForeignKey(MediaFile, verbose_name=_('media file'))
    url = models.URLField(_('URL'))
    type = models.CharField(_('type'), max_length=20, choices=TYPE_CHOICES)
    code = models.CharField(
        _('code'), max_length=40, default=generate_key, unique=True)

    active_from = models.DateTimeField(
        _('active from'), default=timezone.now)
    active_until = models.DateTimeField(
        _('active until'), blank=True, null=True)

    embeds = models.PositiveIntegerField(
        _('embeds'), default=0, editable=False,
        help_text=_(
            'How many times has this banner been embdedded on a website?'))
    impressions = models.PositiveIntegerField(
        _('impressions'), default=0,
        editable=False,
        help_text=_(
            'How many times has an impression been registered using'
            ' a Javascript callback, verifying that it actually was a'
            ' browser? (Too low because of network issues and deactivated'
            ' Javascript support in some browsers.)'))

    objects = BannerManager()

    class Meta:
        ordering = ['-active_from']
        verbose_name = _('banner')
        verbose_name_plural = _('banners')

    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return ('banner_click', (), {'code': self.code})

    @models.permalink
    def impression_url(self):
        return ('banner_impression', (), {'code': self.code})

    def click(self, request):
        x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
        if x_forwarded_for:
            ip = x_forwarded_for.split(',')[0]
        else:
            ip = request.META.get('REMOTE_ADDR')

        self.clicks.create(
            ip=ip or None,
            user_agent=request.META.get('HTTP_USER_AGENT', ''),
            referrer=request.META.get('HTTP_REFERER', ''),
        )

    def click_count(self):
        return self.clicks.count()
    click_count.short_description = _('click count')


class Click(models.Model):
    banner = models.ForeignKey(
        Banner, verbose_name=_('banner'), related_name='clicks')
    timestamp = models.DateTimeField(_('timestamp'), default=timezone.now)
    ip = models.IPAddressField(_('IP'), blank=True, null=True)
    user_agent = models.TextField(_('user agent'), blank=True, default='')
    referrer = models.TextField(_('referrer'), blank=True, default='')

    class Meta:
        ordering = ['-timestamp']
        verbose_name = _('click')
        verbose_name_plural = _('clicks')



from __future__ import absolute_import, unicode_literals

from django.contrib import admin

from feincms_banners import models


admin.site.register(
    models.Banner,
    list_display=(
        'name', 'is_active', 'type', 'url', 'active_from',
        'active_until', 'embeds', 'impressions', 'click_count'),
    list_filter=('is_active', 'type'),
    raw_id_fields=('mediafile',),
    search_fields=('name', 'url', 'code'),
)
admin.site.register(
    models.Click,
    list_display=('timestamp', 'banner', 'ip', 'user_agent', 'referrer'),
    search_fields=('banner__name', 'user_agent', 'referrer'),
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone
import feincms_banners.models
import feincms.module.medialibrary.fields


class Migration(migrations.Migration):

    dependencies = [
        ('medialibrary', '__latest__'),
    ]

    operations = [
        migrations.CreateModel(
            name='Banner',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('is_active', models.BooleanField(default=True, verbose_name='is active')),
                ('name', models.CharField(help_text='Only for internal use, will not be shown on the website.', max_length=100, verbose_name='name')),
                ('url', models.URLField(verbose_name='URL')),
                ('type', models.CharField(max_length=20, verbose_name='type', choices=[('skyscraper', 'skyscraper'), ('leaderboard', 'leaderboard'), ('box', 'box')])),
                ('code', models.CharField(default=feincms_banners.models.generate_key, unique=True, max_length=40, verbose_name='code')),
                ('active_from', models.DateTimeField(default=django.utils.timezone.now, verbose_name='active from')),
                ('active_until', models.DateTimeField(null=True, verbose_name='active until', blank=True)),
                ('embeds', models.PositiveIntegerField(default=0, help_text='How many times has this banner been embdedded on a website?', verbose_name='embeds', editable=False)),
                ('impressions', models.PositiveIntegerField(default=0, help_text='How many times has an impression been registered using a Javascript callback, verifying that it actually was a browser? (Too low because of network issues and deactivated Javascript support in some browsers.)', verbose_name='impressions', editable=False)),
                ('mediafile', feincms.module.medialibrary.fields.MediaFileForeignKey(verbose_name='media file', to='medialibrary.MediaFile')),
            ],
            options={
                'ordering': ['-active_from'],
                'verbose_name': 'banner',
                'verbose_name_plural': 'banners',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Click',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('timestamp', models.DateTimeField(default=django.utils.timezone.now, verbose_name='timestamp')),
                ('ip', models.IPAddressField(null=True, verbose_name='IP', blank=True)),
                ('user_agent', models.TextField(default='', verbose_name='user agent', blank=True)),
                ('referrer', models.TextField(default='', verbose_name='referrer', blank=True)),
                ('banner', models.ForeignKey(verbose_name='banner', to='feincms_banners.Banner')),
            ],
            options={
                'ordering': ['-timestamp'],
                'verbose_name': 'click',
                'verbose_name_plural': 'clicks',
            },
            bases=(models.Model,),
        ),
    ]
